package tech.riemann.ims.entity.acl;

import org.nutz.dao.entity.annotation.ColDefine;
import org.nutz.dao.entity.annotation.Column;
import org.nutz.dao.entity.annotation.Comment;
import org.nutz.dao.entity.annotation.Table;
import org.nutz.json.JsonField;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonGetter;

import club.zhcs.lina.utils.enums.Codebook;
import club.zhcs.lina.utils.enums.ICodeBook;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.media.Schema.RequiredMode;
import lombok.AllArgsConstructor;
import lombok.Builder.Default;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import tech.riemann.ims.entity.IdBaseEntity;

import java.io.Serial;

/**
 * 权限
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-07-22 13:56:54
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@TableName(value = "t_permission", autoResultMap = true)
@Table("t_permission")
@Comment("权限")
@Schema(name = "Permission", description = "权限")
public class Permission extends IdBaseEntity {

    @Serial
    private static final long serialVersionUID = 1L;

    @Schema(description = "权限key,英文", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("p_key")
    @Column("p_key")
    @Comment("权限key,英文")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String key;

    @Schema(description = "权限keyPath,用来做业务,(父级keyPath.key)", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("p_key_path")
    @Column("p_key_path")
    @Comment("权限keyPath,用来做业务,(父级keyPath.key)")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String keyPath;

    @Schema(description = "权限名称,中文用来做标识", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("p_name")
    @Column("p_name")
    @Comment("权限名称,中文用来做标识")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String name;

    @Schema(description = "权限描述", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("p_description")
    @Column("p_description")
    @Comment("权限描述")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String description;

    @Schema(description = "父权限key", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("p_parent_key")
    @Column("p_parent_key")
    @Comment("父权限key")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String parentKey;

    @Schema(description = "权限类型", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("p_type")
    @Column("p_type")
    @Comment("权限类型")
    @ColDefine(notNull = false, width = 20, precision = 0)
    @Default
    private Type type = Type.MENU;

    @Getter
    @AllArgsConstructor
    public enum Type implements ICodeBook {
        /**
         * 菜单/页面
         */
        MENU("menu", "菜单"),
        /**
         * 按钮
         */
        BUTTON("button", "按钮"),
        /**
         * 其他页面元素
         */
        OTHER("other", "其他页面元素");

        String code;
        String description;

    }

    @JsonGetter
    @JsonField
    public Codebook getTypeInfo() {
        return type == null ? null : type.build();
    }

    public void setTypeInfo(Codebook typeInfo) {
        setType(Type.valueOf(typeInfo.getName()));
    }
}
