package tech.riemann.ims.entity.acl;

import org.nutz.dao.entity.annotation.ColDefine;
import org.nutz.dao.entity.annotation.Column;
import org.nutz.dao.entity.annotation.Comment;
import org.nutz.dao.entity.annotation.Table;

import com.baomidou.mybatisplus.annotation.TableField;

import club.zhcs.lina.utils.enums.ICodeBook;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.media.Schema.RequiredMode;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import tech.riemann.ims.entity.IdBaseEntity;

import java.io.Serial;

/**
 * 用户三方账号绑定
 *
 * @author Kerbores(kerbores@riemann.tech)
 *
 * @since 2023-12-15 22:56:03
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@Table("t_user_bind")
@Comment("用户三方账号绑定")
@Schema(name = "UserBind", description = "用户三方账号绑定")
public class UserBind extends IdBaseEntity {

    @Serial
    private static final long serialVersionUID = 1L;

    @Schema(description = "用户名", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("b_user_name")
    @Column("b_user_name")
    @Comment("用户名")
    @ColDefine(notNull = false, width = 50, precision = 0)
    private String userName;

    @Schema(description = "OAuth2 渠道类型", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("b_type")
    @Column("b_type")
    @Comment("OAuth2 渠道类型")
    @ColDefine(notNull = false, width = 20, precision = 0)
    private Type type;

    @Schema(description = "openid", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("b_openid")
    @Column("b_openid")
    @Comment("openid")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String openid;

    @Schema(description = "metadata", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("b_metadata")
    @Column("b_metadata")
    @Comment("metadata")
    @ColDefine(notNull = false, width = 3000, precision = 0)
    private String metadata;

    @Schema(description = "uuid", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("b_uuid")
    @Column("b_uuid")
    @Comment("uuid")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String uuid;

    @Getter
    @AllArgsConstructor
    public enum Type implements ICodeBook {

        WORK_WECHAT("WORK_WECHAT", "企业微信"),
        /**
         * 
         */
        AD("AD", "AD域账号"),
        /**
         * 
         */
        GITEA("gitea", "GITEA");

        String code;
        String description;

    }
}
